@@ -55,6 +55,28 @@ bool network_is_connected(const char *path)
 	return network->connected;
 }
 
+const char *network_get_type(const char *path)
+{
+	const struct network *network;
+	const struct proxy_interface *proxy =
+			proxy_interface_find(IWD_NETWORK_INTERFACE, path);
+
+	network = proxy_interface_get_data(proxy);
+
+	return network->type;
+}
+
+const char *network_get_name(const char *path)
+{
+	const struct network *network;
+	const struct proxy_interface *proxy =
+			proxy_interface_find(IWD_NETWORK_INTERFACE, path);
+
+	network = proxy_interface_get_data(proxy);
+
+	return network->name;
+}
+
 void network_connect(const struct proxy_interface *proxy)
 {
 	if (!proxy)
@@ -64,87 +86,6 @@ void network_connect(const struct proxy_interface *proxy)
 						check_errors_method_callback);
 }
 
-static const char *skip_spaces(const char *p)
-{
-	while (*p == ' ')
-		p++;
-
-	return p;
-}
-
-struct network_args *network_parse_args(const char *args)
-{
-	struct network_args *network_args;
-	char **arg_arr;
-	size_t quoted_len = 0;
-	size_t i;
-	const char *p;
-
-	if (unlikely(!args))
-		return NULL;
-
-	if (args[0] == '\0')
-		return NULL;
-
-	args = skip_spaces(args);
-	p = args;
-
-	network_args = l_new(struct network_args, 1);
-
-	if (*p == '"') {
-		for (++p, i = 0; *p; p++) {
-			i++;
-
-			if (*p != '"')
-				continue;
-
-			quoted_len = i;
-		}
-
-		if (!quoted_len) {
-			p = args;
-			goto split;
-		}
-
-		network_args->name = l_strndup(args + 1, quoted_len - 1);
-		p = args + quoted_len + 1;
-		p = skip_spaces(p);
-	}
-split:
-
-	arg_arr = l_strsplit(p, ' ');
-	if (!arg_arr || !arg_arr[0])
-		goto done;
-
-	if (quoted_len) {
-		network_args->type = l_strdup(arg_arr[0]);
-	} else {
-		network_args->name = l_strdup(arg_arr[0]);
-
-		i = 1;
-
-		while (arg_arr[i] && *arg_arr[i] == '\0')
-			i++;
-
-		network_args->type = l_strdup(arg_arr[i]);
-	}
-
-done:
-	l_strfreev(arg_arr);
-
-	return network_args;
-}
-
-void network_args_destroy(struct network_args *network_args)
-{
-	if (unlikely(!network_args))
-		return;
-
-	l_free(network_args->name);
-	l_free(network_args->type);
-	l_free(network_args);
-}
-
 static const char *get_name(const void *data)
 {
 	const struct network *network = data;
@@ -152,7 +93,7 @@ static const char *get_name(const void *data)
 	return network->name;
 }
 
-static void set_name(void *data, struct l_dbus_message_iter *variant)
+static void update_name(void *data, struct l_dbus_message_iter *variant)
 {
 	struct network *network = data;
 	const char *value;
@@ -168,7 +109,7 @@ static void set_name(void *data, struct l_dbus_message_iter *variant)
 	network->name = l_strdup(value);
 }
 
-static void set_connected(void *data, struct l_dbus_message_iter *variant)
+static void update_connected(void *data, struct l_dbus_message_iter *variant)
 {
 	struct network *network = data;
 	bool value;
@@ -182,7 +123,7 @@ static void set_connected(void *data, struct l_dbus_message_iter *variant)
 	network->connected = value;
 }
 
-static void set_device(void *data, struct l_dbus_message_iter *variant)
+static void update_device(void *data, struct l_dbus_message_iter *variant)
 {
 	struct network *network = data;
 	const char *path;
@@ -196,7 +137,7 @@ static void set_device(void *data, struct l_dbus_message_iter *variant)
 	network->device = proxy_interface_find(IWD_DEVICE_INTERFACE, path);
 }
 
-static void set_type(void *data, struct l_dbus_message_iter *variant)
+static void update_type(void *data, struct l_dbus_message_iter *variant)
 {
 	struct network *network = data;
 	const char *value;
@@ -213,10 +154,10 @@ static void set_type(void *data, struct l_dbus_message_iter *variant)
 }
 
 static const struct proxy_interface_property network_properties[] = {
-	{ "Name",       "s", set_name, get_name },
-	{ "Connected",  "b", set_connected},
-	{ "Device",     "o", set_device},
-	{ "Type",       "s", set_type},
+	{ "Name",       "s", update_name, get_name },
+	{ "Connected",  "b", update_connected},
+	{ "Device",     "o", update_device},
+	{ "Type",       "s", update_type},
 	{ }
 };
 
@@ -306,7 +247,7 @@ char *network_name_completion(const struct proxy_interface *device,
 
 	return proxy_property_str_completion(&network_interface_type,
 						match_by_partial_name, "Name",
-						&params, state);
+						&params, state, NULL);
 }
 
 struct network_search_parameters {
